@@ -65,19 +65,20 @@ opts = {
         'excludes': [
             'docutils'
         ],
+		  'optimize': 2,
     }
 }
 
 
 setup(
     name = 'Gajim',
-    version = '0.12.5',
+    version = '0.13-rc5',
     description = 'A full featured Jabber client',
     author = 'Gajim Development Team',
     url = 'http://www.gajim.org/',
     download_url = 'http://www.gajim.org/downloads.php',
     license = 'GPL',
-    
+
     windows = [{'script': 'src/gajim.py',
                 'icon_resources': [(1, 'data/pixmaps/gajim.ico')]},
                {'script': 'src/history_manager.py',
