@@ -178,6 +178,17 @@ class Contacts:
 		del self._gc_contacts[old_name]
 		del self._metacontacts_tags[old_name]
 
+	def change_contact_jid(self, old_jid, new_jid, account):
+		if account not in self._contacts:
+			return
+		if old_jid not in self._contacts[account]:
+			return
+		self._contacts[account][new_jid] = []
+		for _contact in self._contacts[account][old_jid]:
+			_contact.jid = new_jid
+			self._contacts[account][new_jid].append(_contact)
+		del self._contacts[account][old_jid]
+
 	def add_account(self, account):
 		self._contacts[account] = {}
 		self._gc_contacts[account] = {}
@@ -197,7 +208,7 @@ class Contacts:
 		caps_hash_method=None, caps_hash=None, our_chatstate=None,
 		chatstate=None, last_status_time=None, composing_xep=None,
 		mood={}, tune={}, activity={}):
-	
+
 		# We don't want duplicated group values
 		groups_unique = []
 		for group in groups:
@@ -210,7 +221,7 @@ class Contacts:
 			caps_hash=caps_hash, our_chatstate=our_chatstate, chatstate=chatstate,
 			last_status_time=last_status_time, composing_xep=composing_xep,
 			mood=mood, tune=tune, activity=activity)
-	
+
 	def copy_contact(self, contact):
 		return self.create_contact(jid=contact.jid, name=contact.name,
 			groups=contact.groups, show=contact.show, status=contact.status,
@@ -272,7 +283,7 @@ class Contacts:
 			return self._contacts[account][jid]
 		else:
 			return []
-	
+
 	def get_contact(self, account, jid, resource=None):
 		### WARNING ###
 		# This function returns a *RANDOM* resource if resource = None!
@@ -379,6 +390,12 @@ class Contacts:
 		#FIXME: can this append ?
 		assert False
 
+	def get_metacontacts_tags(self, account):
+		'''return a list of tags for a given account'''
+		if not account in self._metacontacts_tags:
+			return []
+		return self._metacontacts_tags[account].keys()
+
 	def get_metacontacts_tag(self, account, jid):
 		'''Returns the tag of a jid'''
 		if not account in self._metacontacts_tags:
@@ -389,7 +406,7 @@ class Contacts:
 					return tag
 		return None
 
-	def add_metacontact(self, brother_account, brother_jid, account, jid):
+	def add_metacontact(self, brother_account, brother_jid, account, jid, order=None):
 		tag = self.get_metacontacts_tag(brother_account, brother_jid)
 		if not tag:
 			tag = self.get_new_metacontacts_tag(brother_jid)
@@ -406,8 +423,12 @@ class Contacts:
 		if tag not in self._metacontacts_tags[account]:
 			self._metacontacts_tags[account][tag] = [{'jid': jid, 'tag': tag}]
 		else:
-			self._metacontacts_tags[account][tag].append({'jid': jid,
-				'tag': tag})
+			if order:
+				self._metacontacts_tags[account][tag].append({'jid': jid,
+					'tag': tag, 'order': order})
+			else:
+				self._metacontacts_tags[account][tag].append({'jid': jid,
+					'tag': tag})
 		common.gajim.connections[account].store_metacontacts(
 			self._metacontacts_tags[account])
 
@@ -437,7 +458,7 @@ class Contacts:
 			nearby_family = [data for data in family
 				if account in accounts]
 			bb_data = self.get_metacontacts_big_brother(nearby_family)
-			if bb_data['jid'] == jid and bb_data['account'] == account: 
+			if bb_data['jid'] == jid and bb_data['account'] == account:
 				return True
 		return False
 
@@ -458,6 +479,9 @@ class Contacts:
 		[{'account': acct, 'jid': jid, 'order': order}, ]
 		'order' is optional'''
 		tag = self.get_metacontacts_tag(account, jid)
+		return self.get_metacontacts_family_from_tag(account, tag)
+
+	def get_metacontacts_family_from_tag(self, account, tag):
 		if not tag:
 			return []
 		answers = []
@@ -484,7 +508,7 @@ class Contacts:
 		# is not in our roster
 		if not contact1:
 			if contact2:
-				return -1 # prefer the known contact 	
+				return -1 # prefer the known contact
 			else:
 				show1 = 0
 				priority1 = 0
@@ -510,6 +534,10 @@ class Contacts:
 				return 1
 			if data1['order'] < data2['order']:
 				return -1
+		if 'order' in data1:
+			return 1
+		if 'order' in data2:
+			return -1
 		transport1 = common.gajim.get_transport_name_from_jid(jid1)
 		transport2 = common.gajim.get_transport_name_from_jid(jid2)
 		if transport2 and not transport1:
@@ -568,9 +596,9 @@ class Contacts:
 	def contact_from_gc_contact(self, gc_contact):
 		'''Create a Contact instance from a GC_Contact instance'''
 		jid = gc_contact.get_full_jid()
-		return Contact(jid=jid, resource='', name=gc_contact.name, groups=[],
-			show=gc_contact.show, status=gc_contact.status, sub='none',
-			caps_node=gc_contact.caps_node,
+		return Contact(jid=jid, resource=gc_contact.resource,
+			name=gc_contact.name, groups=[], show=gc_contact.show,
+			status=gc_contact.status, sub='none', caps_node=gc_contact.caps_node,
 			caps_hash_method=gc_contact.caps_hash_method,
 			caps_hash=gc_contact.caps_hash)
 
@@ -578,7 +606,7 @@ class Contacts:
 		role='', affiliation='', jid='', resource=''):
 		return GC_Contact(room_jid, name, show, status, role, affiliation, jid,
 			resource)
-	
+
 	def add_gc_contact(self, account, gc_contact):
 		# No such account before ?
 		if account not in self._gc_contacts:
