@@ -23,13 +23,11 @@
 ## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 ##
 
-import re
-
 docdir = '../'
 datadir = '../'
 localedir = '../po'
 
-version = '0.12.5'
+version = '0.13'
 
 import sys, os.path
 for base in ('.', 'common'):
