@@ -55,7 +55,7 @@ from common import i18n
 from common import gajim
 from common import helpers
 
-gtk.glade.bindtextdomain(i18n.APP, i18n.DIR) 
+gtk.glade.bindtextdomain(i18n.APP, i18n.DIR)
 gtk.glade.textdomain(i18n.APP)
 
 screen_w = gtk.gdk.screen_width()
@@ -71,11 +71,11 @@ def get_completion_liststore(entry):
 	completion list consists of (Pixbuf, Text) rows'''
 	completion = gtk.EntryCompletion()
 	liststore = gtk.ListStore(gtk.gdk.Pixbuf, str)
-	
+
 	render_pixbuf = gtk.CellRendererPixbuf()
 	completion.pack_start(render_pixbuf, expand = False)
 	completion.add_attribute(render_pixbuf, 'pixbuf', 0)
-	
+
 	render_text = gtk.CellRendererText()
 	completion.pack_start(render_text, expand = True)
 	completion.add_attribute(render_text, 'text', 1)
@@ -83,21 +83,21 @@ def get_completion_liststore(entry):
 	completion.set_model(liststore)
 	entry.set_completion(completion)
 	return liststore
-	
-	
+
+
 def popup_emoticons_under_button(menu, button, parent_win):
 	''' pops emoticons menu under button, which is in parent_win'''
 	window_x1, window_y1 = parent_win.get_origin()
 	def position_menu_under_button(menu):
 		# inline function, which will not keep refs, when used as CB
 		button_x, button_y = button.allocation.x, button.allocation.y
-		
+
 		# now convert them to X11-relative
 		window_x, window_y = window_x1, window_y1
 		x = window_x + button_x
 		y = window_y + button_y
 
-		menu_width, menu_height = menu.size_request()
+		menu_height = menu.size_request()[1]
 
 		## should we pop down or up?
 		if (y + button.allocation.height + menu_height
@@ -113,7 +113,7 @@ def popup_emoticons_under_button(menu, button, parent_win):
 		return (x, y, push_in)
 
 	menu.popup(None, None, position_menu_under_button, 1, 0)
-	
+
 def get_theme_font_for_option(theme, option):
 	'''return string description of the font, stored in
 	theme preferences'''
@@ -128,12 +128,12 @@ def get_theme_font_for_option(theme, option):
 	fd = pango.FontDescription(font_name)
 	fd.merge(font_desc, True)
 	return fd.to_string()
-	
+
 def get_default_font():
 	'''Get the desktop setting for application font
 	first check for GNOME, then Xfce and last KDE
 	it returns None on failure or else a string 'Font Size' '''
-	
+
 	try:
 		import gconf
 		# in try because daemon may not be there
@@ -150,11 +150,11 @@ def get_default_font():
 	# and http://freedesktop.org/Standards/basedir-spec
 	xdg_config_home = os.environ.get('XDG_CONFIG_HOME', '')
 	if xdg_config_home == '':
-		xdg_config_home = os.path.expanduser('~/.config') # default	
+		xdg_config_home = os.path.expanduser('~/.config') # default
 	xfce_config_file = os.path.join(xdg_config_home, 'xfce4/mcs_settings/gtk.xml')
-	
+
 	kde_config_file = os.path.expanduser('~/.kde/share/config/kdeglobals')
-	
+
 	if os.path.exists(xfce_config_file):
 		try:
 			for line in open(xfce_config_file):
@@ -164,7 +164,7 @@ def get_default_font():
 		except Exception:
 			#we talk about file
 			print >> sys.stderr, _('Error: cannot open %s for reading') % xfce_config_file
-	
+
 	elif os.path.exists(kde_config_file):
 		try:
 			for line in open(kde_config_file):
@@ -179,9 +179,9 @@ def get_default_font():
 		except Exception:
 			#we talk about file
 			print >> sys.stderr, _('Error: cannot open %s for reading') % kde_config_file
-	
+
 	return None
-	
+
 def autodetect_browser_mailer():
 	# recognize the environment and set appropriate browser/mailer
 	if user_runs_gnome():
@@ -190,8 +190,6 @@ def autodetect_browser_mailer():
 		gajim.config.set('openwith', 'kfmclient exec')
 	elif user_runs_xfce():
 		gajim.config.set('openwith', 'exo-open')
-	elif user_runs_osx():
-		gajim.config.set('openwith', 'open')
 	else:
 		gajim.config.set('openwith', 'custom')
 
@@ -207,9 +205,6 @@ def user_runs_xfce():
 		return True
 	return False
 
-def user_runs_osx():
-	return sys.platform == 'darwin'
-
 def get_running_processes():
 	'''returns running processes or None (if not /proc exists)'''
 	if os.path.isdir('/proc'):
@@ -229,19 +224,19 @@ def get_running_processes():
 		files = filter(str.isdigit, files)
 
 		# files that aren't directories...
-		files = filter(lambda f:os.path.isdir('/proc/' + f), files)
+		files = [f for f in files if os.path.isdir('/proc/' + f)]
 
 		# processes owned by somebody not running gajim...
 		# (we check if we have access to that file)
-		files = filter(lambda f:os.access('/proc/' + f +'/exe', os.F_OK), files)
+		files = [f for f in files if os.access('/proc/' + f +'/exe', os.F_OK)]
 
 		# be sure that /proc/[number]/exe is really a symlink
 		# to avoid TBs in incorrectly configured systems
-		files = filter(lambda f:os.path.islink('/proc/' + f + '/exe'), files)
+		files = [f for f in files if os.path.islink('/proc/' + f + '/exe')]
 
 		# list of processes
 		processes = [os.path.basename(os.readlink('/proc/' + f +'/exe')) for f in files]
-		
+
 		return processes
 	return []
 
@@ -371,17 +366,17 @@ def get_abspath_for_script(scriptname, want_type = False):
 	'''checks if we are svn or normal user and returns abspath to asked script
 	if want_type is True we return 'svn' or 'install' '''
 	if os.path.isdir('.svn'): # we are svn user
-		type = 'svn'
+		type_ = 'svn'
 		cwd = os.getcwd() # it's always ending with src
 
 		if scriptname == 'gajim-remote':
 			path_to_script = cwd + '/gajim-remote.py'
-		
+
 		elif scriptname == 'gajim':
 			script = '#!/bin/sh\n' # the script we may create
 			script += 'cd %s' % cwd
 			path_to_script = cwd + '/../scripts/gajim_sm_script'
-				
+
 			try:
 				if os.path.exists(path_to_script):
 					os.remove(path_to_script)
@@ -397,13 +392,13 @@ def get_abspath_for_script(scriptname, want_type = False):
 				print >> sys.stderr, s
 
 	else: # normal user (not svn user)
-		type = 'install'
+		type_ = 'install'
 		# always make it like '/usr/local/bin/gajim'
 		path_to_script = helpers.is_in_path(scriptname, True)
-		
-	
+
+
 	if want_type:
-		return path_to_script, type
+		return path_to_script, type_
 	else:
 		return path_to_script
 
@@ -458,7 +453,7 @@ def possibly_move_window_in_current_desktop(window):
 	root_window = gtk.gdk.screen_get_default().get_root_window()
 	# current user's vd
 	current_virtual_desktop_no = get_current_desktop(root_window)
-	
+
 	# vd roster window is in
 	window_virtual_desktop = get_current_desktop(window.window)
 
@@ -476,13 +471,13 @@ def file_is_locked(path_to_file):
 	'''returns True if file is locked (WINDOWS ONLY)'''
 	if os.name != 'nt': # just in case
 		return
-	
+
 	if not HAS_PYWIN32:
 		return
-	
+
 	secur_att = pywintypes.SECURITY_ATTRIBUTES()
 	secur_att.Initialize()
-	
+
 	try:
 		# try make a handle for READING the file
 		hfile = win32file.CreateFile(
@@ -494,7 +489,7 @@ def file_is_locked(path_to_file):
 			win32con.FILE_ATTRIBUTE_NORMAL,	# normal file
 			0								# no attr. template
 		)
-	except pywintypes.error, e:
+	except pywintypes.error:
 		return True
 	else: # in case all went ok, close file handle (go to hell WinAPI)
 		hfile.Close()
@@ -523,7 +518,7 @@ def _get_fade_color(treeview, selected, focused):
 def get_scaled_pixbuf(pixbuf, kind):
 	'''returns scaled pixbuf, keeping ratio etc or None
 	kind is either "chat", "roster", "notification", "tooltip", "vcard"'''
-	
+
 	# resize to a width / height for the avatar not to have distortion
 	# (keep aspect ratio)
 	width = gajim.config.get(kind + '_avatar_width')
@@ -629,7 +624,7 @@ def get_path_to_generic_or_avatar(generic, jid = None, suffix = None):
 		for extension in ('.png', '.jpeg'):
 			path_to_local_file_full = path_to_local_file + extension
 			if os.path.exists(path_to_local_file_full):
-				return path_to_local_file_full 
+				return path_to_local_file_full
 		for extension in ('.png', '.jpeg'):
 			path_to_file_full = path_to_file + extension
 			if os.path.exists(path_to_file_full):
@@ -641,7 +636,7 @@ def decode_filechooser_file_paths(file_paths):
 	ask sys.getfilesystemencoding() in POSIX
 	file_paths MUST be LIST'''
 	file_paths_list = list()
-	
+
 	if os.name == 'nt': # decode as UTF-8 under Windows
 		for file_path in file_paths:
 			file_path = file_path.decode('utf8')
@@ -656,14 +651,14 @@ def decode_filechooser_file_paths(file_paths):
 				except Exception:
 					pass
 			file_paths_list.append(file_path)
-	
+
 	return file_paths_list
 
 def possibly_set_gajim_as_xmpp_handler():
 	'''registers (by default only the first time) xmmp: to Gajim.'''
 	path_to_dot_kde = os.path.expanduser('~/.kde')
 	if os.path.exists(path_to_dot_kde):
-		path_to_kde_file = os.path.join(path_to_dot_kde, 
+		path_to_kde_file = os.path.join(path_to_dot_kde,
 			'share/services/xmpp.protocol')
 	else:
 		path_to_kde_file = None
@@ -795,7 +790,7 @@ default_name = ''):
 		# Save image
 		try:
 			pixbuf.save(file_path, type_)
-		except:
+		except Exception:
 			if os.path.exists(file_path):
 				os.remove(file_path)
 			new_file_path = '.'.join(file_path.split('.')[:-1]) + '.jpeg'
@@ -808,17 +803,13 @@ default_name = ''):
 			dialog.destroy()
 
 	def on_ok(widget):
-		def on_ok2(file_path, pixbuf):
-			pixbuf.save(file_path, 'jpeg')
-			dialog.destroy()
-
 		file_path = dialog.get_filename()
 		file_path = decode_filechooser_file_paths((file_path,))[0]
 		if os.path.exists(file_path):
 			# check if we have write permissions
 			if not os.access(file_path, os.W_OK):
 				file_name = os.path.basename(file_path)
-				dialogs.ErrorDialog(_('Cannot overwrite existing file "%s"' % 
+				dialogs.ErrorDialog(_('Cannot overwrite existing file "%s"' %
 					file_name),
 				_('A file with this name already exists and you do not have '
 				'permission to overwrite it.'))
@@ -841,7 +832,7 @@ default_name = ''):
 	def on_cancel(widget):
 		dialog.destroy()
 
-	dialog = dialogs.FileChooserDialog(title_text=_('Save Image as...'), 
+	dialog = dialogs.FileChooserDialog(title_text=_('Save Image as...'),
 		action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_CANCEL,
 		gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK),
 		default_response=gtk.RESPONSE_OK,
@@ -872,21 +863,39 @@ def create_combobox(value_list, selected_value = None):
 	combobox.show_all()
 	return combobox
 
-def load_iconset(path, pixbuf2 = None, transport = False):
+def create_list_multi(value_list, selected_values=None):
+	'''Value_list is [(label1, value1), ]'''
+	liststore = gtk.ListStore(str, str)
+	treeview = gtk.TreeView(liststore)
+	treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+	treeview.set_headers_visible(False)
+	col = gtk.TreeViewColumn()
+	treeview.append_column(col)
+	cell = gtk.CellRendererText()
+	col.pack_start(cell, True)
+	col.set_attributes(cell, text=0)
+	for value in value_list:
+		iter = liststore.append(value)
+		if value[1] in selected_values:
+			treeview.get_selection().select_iter(iter)
+	treeview.show_all()
+	return treeview
+
+def load_iconset(path, pixbuf2=None, transport=False):
 	'''load full iconset from the given path, and add
 	pixbuf2 on top left of each static images'''
 	path += '/'
 	if transport:
-		list = ('online', 'chat', 'away', 'xa', 'dnd', 'offline',
+		list_ = ('online', 'chat', 'away', 'xa', 'dnd', 'offline',
 			'not in roster')
 	else:
-		list = ('connecting', 'online', 'chat', 'away', 'xa', 'dnd',
+		list_ = ('connecting', 'online', 'chat', 'away', 'xa', 'dnd',
 			'invisible', 'offline', 'error', 'requested', 'event', 'opened',
 			'closed', 'not in roster', 'muc_active', 'muc_inactive')
 		if pixbuf2:
-			list = ('connecting', 'online', 'chat', 'away', 'xa', 'dnd',
+			list_ = ('connecting', 'online', 'chat', 'away', 'xa', 'dnd',
 				'offline', 'error', 'requested', 'event', 'not in roster')
-	return _load_icon_list(list, path, pixbuf2)
+	return _load_icon_list(list_, path, pixbuf2)
 
 def load_icon(icon_name):
 	'''load an icon from the iconset in 16x16'''
@@ -948,9 +957,9 @@ def _load_icon_list(icons_list, path, pixbuf2 = None):
 		image = gtk.Image()
 		image.show()
 		imgs[icon] = image
-		for file in files: # loop seeking for either gif or png
-			if os.path.exists(file):
-				image.set_from_file(file)
+		for file_ in files: # loop seeking for either gif or png
+			if os.path.exists(file_):
+				image.set_from_file(file_)
 				if pixbuf2 and image.get_storage_type() == gtk.IMAGE_PIXBUF:
 					# add pixbuf2 on top-left corner of image
 					pixbuf1 = image.get_pixbuf()
@@ -966,11 +975,17 @@ def make_jabber_state_images():
 	'''initialise jabber_state_images dict'''
 	iconset = gajim.config.get('iconset')
 	if iconset:
-		path = os.path.join(helpers.get_iconset_path(iconset), '16x16')
-		if not os.path.exists(path):
+		if helpers.get_iconset_path(iconset):
+			path = os.path.join(helpers.get_iconset_path(iconset), '16x16')
+			if not os.path.exists(path):
+				iconset = gajim.config.DEFAULT_ICONSET
+				gajim.config.set('iconset', iconset)
+		else:
 			iconset = gajim.config.DEFAULT_ICONSET
+			gajim.config.set('iconset', iconset)
 	else:
 		iconset = gajim.config.DEFAULT_ICONSET
+		gajim.config.set('iconset', iconset)
 
 	path = os.path.join(helpers.get_iconset_path(iconset), '32x32')
 	gajim.interface.jabber_state_images['32'] = load_iconset(path)
@@ -986,4 +1001,31 @@ def reload_jabber_state_images():
 	make_jabber_state_images()
 	gajim.interface.roster.update_jabber_state_images()
 
+def label_set_autowrap(widget):
+	'''Make labels automatically re-wrap if their containers are resized.
+	Accepts label or container widgets.'''
+	if isinstance (widget, gtk.Container):
+		children = widget.get_children()
+		for i in xrange (len (children)):
+			label_set_autowrap(children[i])
+	elif isinstance(widget, gtk.Label):
+		widget.set_line_wrap(True)
+		widget.connect_after('size-allocate', __label_size_allocate)
+
+def __label_size_allocate(widget, allocation):
+	'''Callback which re-allocates the size of a label.'''
+	layout = widget.get_layout()
+
+	lw_old, lh_old = layout.get_size()
+	# fixed width labels
+	if lw_old/pango.SCALE == allocation.width:
+		return
+
+	# set wrap width to the pango.Layout of the labels ###
+	layout.set_width (allocation.width * pango.SCALE)
+	lw, lh = layout.get_size ()
+
+	if lh_old != lh:
+		widget.set_size_request (-1, lh / pango.SCALE)
+
 # vim: se ts=3:
