@@ -50,6 +50,8 @@ import operator
 
 if __name__ == '__main__':
 	from common import i18n
+	import common.configpaths
+	common.configpaths.gajimpaths.init(None)
 from common import gajim
 
 import tooltips
@@ -91,77 +93,76 @@ element_styles['tt']  = element_styles['kbd']
 element_styles['i']   = element_styles['em']
 element_styles['b']   = element_styles['strong']
 
-'''
-==========
-  JEP-0071
-==========
-
-This Integration Set includes a subset of the modules defined for 
-XHTML 1.0 but does not redefine any existing modules, nor 
-does it define any new modules. Specifically, it includes the 
-following modules only:
-
-- Structure
-- Text
-  
-  * Block
-    
-    phrasal
-       addr, blockquote, pre
-    Struc
-       div,p
-    Heading
-       h1, h2, h3, h4, h5, h6
-    
-  * Inline
-    
-    phrasal
-       abbr, acronym, cite, code, dfn, em, kbd, q, samp, strong, var
-    structural
-       br, span
-  
-- Hypertext (a)
-- List (ul, ol, dl)
-- Image (img)
-- Style Attribute
-     
-Therefore XHTML-IM uses the following content models:
-
-  Block.mix
-            Block-like elements, e.g., paragraphs
-  Flow.mix
-            Any block or inline elements
-  Inline.mix
-            Character-level elements
-  InlineNoAnchor.class
-			Anchor element 
-  InlinePre.mix
-            Pre element
-
-XHTML-IM also uses the following Attribute Groups:
-
-Core.extra.attrib
-	TBD
-I18n.extra.attrib
-	TBD
-Common.extra
-	style
-
-
-...
-#block level:
-#Heading    h
-#           ( pres           = h1 | h2 | h3 | h4 | h5 | h6 )
-#Block      ( phrasal        = address | blockquote | pre )
-#NOT           ( presentational = hr )
-#           ( structural     = div | p )
-#other:     section
-#Inline     ( phrasal        = abbr | acronym | cite | code | dfn | em | kbd | q | samp | strong | var )
-#NOT        ( presentational =  b  | big | i | small | sub | sup | tt )
-#           ( structural     =  br | span )
-#Param/Legacy    param, font, basefont, center, s, strike, u, dir, menu, isindex
+# ==========
+#   JEP-0071
+# ==========
 #
-'''
+# This Integration Set includes a subset of the modules defined for
+# XHTML 1.0 but does not redefine any existing modules, nor
+# does it define any new modules. Specifically, it includes the
+# following modules only:
+#
+# - Structure
+# - Text
+#
+#   * Block
+#
+#     phrasal
+#        addr, blockquote, pre
+#     Struc
+#        div,p
+#     Heading
+#        h1, h2, h3, h4, h5, h6
+#
+#   * Inline
+#
+#     phrasal
+#        abbr, acronym, cite, code, dfn, em, kbd, q, samp, strong, var
+#     structural
+#        br, span
+#
+# - Hypertext (a)
+# - List (ul, ol, dl)
+# - Image (img)
+# - Style Attribute
+#
+# Therefore XHTML-IM uses the following content models:
+#
+#   Block.mix
+#             Block-like elements, e.g., paragraphs
+#   Flow.mix
+#             Any block or inline elements
+#   Inline.mix
+#             Character-level elements
+#   InlineNoAnchor.class
+# 			Anchor element
+#   InlinePre.mix
+#             Pre element
+#
+# XHTML-IM also uses the following Attribute Groups:
+#
+# Core.extra.attrib
+# 	TBD
+# I18n.extra.attrib
+# 	TBD
+# Common.extra
+# 	style
+#
+#
+# ...
+# block level:
+# Heading    h
+#            ( pres           = h1 | h2 | h3 | h4 | h5 | h6 )
+# Block      ( phrasal        = address | blockquote | pre )
+# NOT           ( presentational = hr )
+#            ( structural     = div | p )
+# other:     section
+# Inline     ( phrasal        = abbr | acronym | cite | code | dfn | em |
+#                               kbd | q | samp | strong | var )
+# NOT        ( presentational =  b  | big | i | small | sub | sup | tt )
+#            ( structural     =  br | span )
+# Param/Legacy    param, font, basefont, center, s, strike, u, dir, menu,
+#                 isindex
 
 BLOCK_HEAD = set(( 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', ))
 BLOCK_PHRASAL = set(( 'address', 'blockquote', 'pre', ))
@@ -185,25 +186,6 @@ for name in BLOCK_HEAD:
 													('font-weight: bold', 'font-style: oblique')[weigth],
 											  )
 
-
-def build_patterns(view, config, interface):
-	# extra, rst does not mark _underline_ or /it/ up
-	# actually <b>, <i> or <u> are not in the JEP-0071, but are seen in the wild
-	basic_pattern = r'(?<!\w|\<|/|:)' r'/[^\s/]' r'([^/]*[^\s/])?' r'/(?!\w|/|:)|'\
-					r'(?<!\w)' r'_[^\s_]' r'([^_]*[^\s_])?' r'_(?!\w)'
-	view.basic_pattern_re = re.compile(basic_pattern)
-	# emoticons
-	emoticons_pattern = ''
-	if config.get('emoticons_theme'):
-		emoticons_pattern = gajim.interface.emot_only
-
-	view.emot_pattern_re = re.compile(emoticons_pattern, re.IGNORECASE)
-	# because emoticons match later (in the string) they need to be after
-	# basic matches that may occur earlier
-	emot_and_basic_pattern = basic_pattern + emoticons_pattern
-	view.emot_and_basic_re = re.compile(emot_and_basic_pattern, re.IGNORECASE)
-
-
 def _parse_css_color(color):
 	'''_parse_css_color(css_color) -> gtk.gdk.Color'''
 	if color.startswith('rgb(') and color.endswith(')'):
@@ -213,8 +195,9 @@ def _parse_css_color(color):
 		return gtk.gdk.color_parse(color)
 
 def style_iter(style):
-	return (map(lambda x:x.strip(),item.split(':', 1)) for item in style.split(';') if len(item.strip()))
-	
+	return ([x.strip() for x in item.split(':', 1)] for item in style.split(';')\
+		if len(item.strip()))
+
 
 class HtmlHandler(xml.sax.handler.ContentHandler):
 	"""A handler to display html to a gtk textview.
@@ -222,11 +205,12 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 	It keeps a stack of "style spans" (start/end element pairs)
 	and a stack of list counters, for nested lists.
 	"""
-	def __init__(self, textview, startiter):
+	def __init__(self, conv_textview, startiter):
 		xml.sax.handler.ContentHandler.__init__(self)
-		self.textbuf = textview.get_buffer()
-		self.textview = textview
+		self.textbuf = conv_textview.tv.get_buffer()
+		self.textview = conv_textview.tv
 		self.iter = startiter
+		self.conv_textview = conv_textview
 		self.text = ''
 		self.starting=True
 		self.preserve = False
@@ -244,48 +228,12 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 		tag.set_property('paragraph-background-gdk', color)
 
 
-	#FIXME: when we migrate to 2.10 rm this
-	if gtk.gtk_version >= (2, 8, 5) or gobject.pygtk_version >= (2, 8, 1):
-
-		def _get_current_attributes(self):
-			attrs = self.textview.get_default_attributes()
-			self.iter.backward_char()
-			self.iter.get_attributes(attrs)
-			self.iter.forward_char()
-			return attrs
-		
-	else:
-		
-		# Workaround http://bugzilla.gnome.org/show_bug.cgi?id=317455
-		def _get_current_style_attr(self, propname, comb_oper=None):
-			tags = [tag for tag in self.styles if tag is not None]
-			tags.reverse()
-			is_set_name = propname + '-set'
-			value = None
-			for tag in tags:
-				if tag.get_property(is_set_name):
-					if value is None:
-						value = tag.get_property(propname)
-						if comb_oper is None:
-							return value
-					else:
-						value = comb_oper(value, tag.get_property(propname))
-			return value
-
-		class _FakeAttrs(object):
-			__slots__ = ('font', 'font_scale')
-
-		def _get_current_attributes(self):
-			attrs = self._FakeAttrs()
-			attrs.font_scale = self._get_current_style_attr('scale',
-															operator.mul)
-			if attrs.font_scale is None:
-				attrs.font_scale = 1.0
-			attrs.font = self._get_current_style_attr('font-desc')
-			if attrs.font is None:
-				attrs.font = self.textview.style.font_desc
-			return attrs
-
+	def _get_current_attributes(self):
+		attrs = self.textview.get_default_attributes()
+		self.iter.backward_char()
+		self.iter.get_attributes(attrs)
+		self.iter.forward_char()
+		return attrs
 
 	def __parse_length_frac_size_allocate(self, textview, allocation,
 										  frac, callback, args):
@@ -299,7 +247,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			sign = cmp(val,0)
 			# limits: 1% to 500%
 			val = sign*max(1,min(abs(val),500))
-			frac = val/100 
+			frac = val/100
 			if font_relative:
 				attrs = self._get_current_attributes()
 				font_size = attrs.font.get_size() / pango.SCALE
@@ -352,7 +300,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 				callback(val, *args)
 			except Exception:
 				warnings.warn('Unable to parse length value "%s"' % value)
-		
+
 	def __parse_font_size_cb(length, tag):
 		tag.set_property('size-points', length/display_resolution)
 	__parse_font_size_cb = staticmethod(__parse_font_size_cb)
@@ -406,7 +354,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			length += styles[-1].get_property(propname)
 		tag.set_property(propname, length)
 	#__frac_length_tag_cb = staticmethod(__frac_length_tag_cb)
-		
+
 	def _parse_style_margin_left(self, tag, value):
 		# block relative
 		self._parse_length(value, False, True, 1, 1000, self.__frac_length_tag_cb,
@@ -453,7 +401,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			warnings.warn('Invalid text-align:%s requested' % value)
 		else:
 			tag.set_property('justification', align)
-	
+
 	def _parse_style_text_decoration(self, tag, value):
 		values = value.split(' ')
 		if 'none' in values:
@@ -471,7 +419,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			warnings.warn('text-decoration:blink not implemented')
 		if 'overline' in values:
 			warnings.warn('text-decoration:overline not implemented')
-	
+
 	def _parse_style_white_space(self, tag, value):
 		if value == 'pre':
 			tag.set_property('wrap_mode', gtk.WRAP_NONE)
@@ -485,7 +433,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			tag.set_property(propname, value)
 		except Exception:
 			gajim.log.warn( "Error with prop: " + propname + " for tag: " + str(tag))
-		
+
 
 	def _parse_style_width(self, tag, value):
 		if value == 'auto':
@@ -497,8 +445,8 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			return
 		self._parse_length(value, False, False, 1, 1000, self.__length_tag_cb,
 						   tag, "height")
-	 
-	
+
+
 	# build a dictionary mapping styles to methods, for greater speed
 	__style_methods = dict()
 	for style in ('background-color', 'color', 'font-family', 'font-size',
@@ -525,7 +473,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			tag.href = href
 			tag.type_ = type_ # to be used by the URL handler
 			tag.connect('event', self.textview.html_hyperlink_handler, 'url', href)
-			tag.set_property('foreground', '#0000ff')
+			tag.set_property('foreground', gajim.config.get('urlmsgcolor'))
 			tag.set_property('underline', pango.UNDERLINE_SINGLE)
 			tag.is_anchor = True
 		if title:
@@ -535,55 +483,54 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 	def _process_img(self, attrs):
 		'''Process a img tag.
 		'''
+		mem = ''
 		try:
-			# Wait maximum 1s for connection 
+			# Wait maximum 1s for connection
 			socket.setdefaulttimeout(1)
-			try: 
-				f = urllib2.urlopen(attrs['src']) 
-			except Exception, ex: 
+			try:
+				f = urllib2.urlopen(attrs['src'])
+			except Exception, ex:
 				gajim.log.debug('Error loading image %s ' % attrs['src']  + str(ex))
-				pixbuf = None 
-				alt = attrs.get('alt', 'Broken image') 
-			else: 
-				# Wait 0.1s between each byte 
-				try: 
-					f.fp._sock.fp._sock.settimeout(0.5) 
-				except Exception: 
-					pass 
-			# Max image size = 2 MB (to try to prevent DoS)
-			mem = ''
-			deadline = time.time() + 3
-			while True:
-				if time.time() > deadline:
-					gajim.log.debug(str('Timeout loading image %s ' % \
-						attrs['src'] + ex))
-					mem = ''
-					alt = attrs.get('alt', '')
-					if alt:
-						alt += '\n'
-					alt += _('Timeout loading image')
-					break
+				pixbuf = None
+				alt = attrs.get('alt', 'Broken image')
+			else:
+				# Wait 0.1s between each byte
 				try:
-					temp = f.read(100)
-				except socket.timeout, ex:
-					gajim.log.debug('Timeout loading image %s ' % attrs['src'] + \
-						str(ex))
-					mem = ''
-					alt = attrs.get('alt', '')
-					if alt:
-						alt += '\n'
-					alt += _('Timeout loading image')
-					break
-				if temp:
-					mem += temp
-				else:
-					break
-				if len(mem) > 2*1024*1024:
-					alt = attrs.get('alt', '')
-					if alt:
-						alt += '\n'
-					alt += _('Image is too big')
-					break
+					f.fp._sock.fp._sock.settimeout(0.5)
+				except Exception:
+					pass
+				# Max image size = 2 MB (to try to prevent DoS)
+				deadline = time.time() + 3
+				while True:
+					if time.time() > deadline:
+						gajim.log.debug(str('Timeout loading image %s ' % \
+							attrs['src'] + ex))
+						mem = ''
+						alt = attrs.get('alt', '')
+						if alt:
+							alt += '\n'
+						alt += _('Timeout loading image')
+						break
+					try:
+						temp = f.read(100)
+					except socket.timeout, ex:
+						gajim.log.debug('Timeout loading image %s ' % attrs['src'] + \
+							str(ex))
+						alt = attrs.get('alt', '')
+						if alt:
+							alt += '\n'
+						alt += _('Timeout loading image')
+						break
+					if temp:
+						mem += temp
+					else:
+						break
+					if len(mem) > 2*1024*1024:
+						alt = attrs.get('alt', '')
+						if alt:
+							alt += '\n'
+						alt += _('Image is too big')
+						break
 			pixbuf = None
 			if mem:
 				# Caveat: GdkPixbuf is known not to be safe to load
@@ -691,7 +638,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 
 	def _starts_line(self):
 		return self.starting or self.iter.starts_line()
-		
+
 	def _flush_text(self):
 		if not self.text: return
 		text, self.text = self.text, ''
@@ -708,52 +655,8 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 		return False
 
 	def handle_specials(self, text):
-		index = 0
-		se = self.textview.config.get('show_ascii_formatting_chars')
-		af = gajim.config.get('ascii_formatting')
-		if self.textview.config.get('emoticons_theme'):
-			if af:
-				iterator = self.textview.emot_and_basic_re.finditer(text)
-			else:
-				iterator = self.textview.emot_pattern_re.finditer(text)
-		elif af:
-			iterator = self.textview.basic_pattern_re.finditer(text)
-		else:
-			iterator = []
-		for match in iterator:
-			start, end = match.span()
-			special_text = text[start:end]
-			if start != 0:
-				self._insert_text(text[index:start])
-			index = end # update index
-			#emoticons
-			possible_emot_ascii_caps = special_text.upper() # emoticons keys are CAPS
-			if self.textview.config.get('emoticons_theme') and \
-					possible_emot_ascii_caps in self.textview.interface.emoticons.keys():
-				#it's an emoticon
-				emot_ascii = possible_emot_ascii_caps
-				anchor = self.textbuf.create_child_anchor(self.iter)
-				img = gtk.Image()
-				img.set_from_file(self.textview.interface.emoticons[emot_ascii])
-				img.show()
-				# TODO: add alt/tooltip with the special_text (a11y) 
-				self.textview.add_child_at_anchor(img, anchor)
-			elif af:
-				# now print it
-				if special_text.startswith('/'): # it's explicit italics
-					self.startElement('i', {})
-				elif special_text.startswith('_'): # it's explicit underline
-					self.startElement('u', {})
-				if se: self._insert_text(special_text[0])
-				self.handle_specials(special_text[1:-1])
-				if se: self._insert_text(special_text[0])
-				if special_text.startswith('_'): # it's explicit underline
-					self.endElement('u')
-				if special_text.startswith('/'): # it's explicit italics
-					self.endElement('i')
-		if index < len(text):
-			self._insert_text(text[index:])
-		
+		self.iter = self.conv_textview.detect_and_print_special_text(text, 							self._get_style_tags())
+
 	def characters(self, content):
 		if self.preserve:
 			self.text += content
@@ -778,7 +681,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 		#FIXME: if we want to use id, it needs to be unique across
 		# the whole textview, so we need to add something like the
 		# message-id to it.
-		#id_ = attrs.get('id',None) 
+		#id_ = attrs.get('id',None)
 		id_ = None
 		if name == 'a':
 			#TODO: accesskey, charset, hreflang, rel, rev, tabindex, type
@@ -801,11 +704,11 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			tag = self._process_img(attrs)
 		if name in element_styles:
 			style += element_styles[name]
-		# so that explicit styles override implicit ones, 
+		# so that explicit styles override implicit ones,
 		# we add the attribute last
 		style += ";"+attrs.get('style','')
 		if style == '':
-			style = None        
+			style = None
 		self._begin_span(style, tag, id_)
 
 		if name == 'br':
@@ -891,7 +794,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 		#    self.text = ' '
 
 class HtmlTextView(gtk.TextView):
-	
+
 	def __init__(self):
 		gobject.GObject.__init__(self)
 		self.set_wrap_mode(gtk.WRAP_CHAR)
@@ -906,7 +809,6 @@ class HtmlTextView(gtk.TextView):
 		self.config = gajim.config
 		self.interface = gajim.interface
 		# end big hack
-		build_patterns(self,gajim.config,gajim.interface)
 
 	def __destroy_event(self, widget):
 		if self.tooltip.timeout != 0:
@@ -941,39 +843,35 @@ class HtmlTextView(gtk.TextView):
 		x, y, _ = widget.window.get_pointer()
 		x, y = widget.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, x, y)
 		tags = widget.get_iter_at_location(x, y).get_tags()
-		is_over_anchor = False
-		for tag in tags:
-			if getattr(tag, 'is_anchor', False):
-				is_over_anchor = True
-				break
+		anchor_tags = [tag for tag in tags if getattr(tag, 'is_anchor', False)]
 		if self.tooltip.timeout != 0:
 			# Check if we should hide the line tooltip
-			if not is_over_anchor:
+			if not anchor_tags:
 				self.tooltip.hide_tooltip()
-		if not self._changed_cursor and is_over_anchor:
+		if not self._changed_cursor and anchor_tags:
 			window = widget.get_window(gtk.TEXT_WINDOW_TEXT)
 			window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
 			self._changed_cursor = True
-			self.tooltip.timeout = gobject.timeout_add(500, self.show_tooltip, tag)
-		elif self._changed_cursor and not is_over_anchor:
+			self.tooltip.timeout = gobject.timeout_add(500, self.show_tooltip, anchor_tags[0])
+		elif self._changed_cursor and not anchor_tags:
 			window = widget.get_window(gtk.TEXT_WINDOW_TEXT)
 			window.set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
 			self._changed_cursor = False
 		return False
 
-	def display_html(self, html):
-		buffer = self.get_buffer()
-		eob = buffer.get_end_iter()
+	def display_html(self, html, conv_textview):
+		buffer_ = self.get_buffer()
+		eob = buffer_.get_end_iter()
 		## this works too if libxml2 is not available
 		# parser = xml.sax.make_parser(['drv_libxml2'])
 		# parser.setFeature(xml.sax.handler.feature_validation, True)
 		parser = xml.sax.make_parser()
-		parser.setContentHandler(HtmlHandler(self, eob))
+		parser.setContentHandler(HtmlHandler(conv_textview, eob))
 		parser.parse(StringIO(html))
-		
+
 		# too much space after :)
 		#if not eob.starts_line():
-		#    buffer.insert(eob, '\n')
+		#    buffer_.insert(eob, '\n')
 
 
 
@@ -981,7 +879,9 @@ change_cursor = None
 
 if __name__ == '__main__':
 	import os
-	from common import gajim
+
+	from conversation_textview import ConversationTextview
+	import gajim as gaj
 
 	class log(object):
 
@@ -994,39 +894,38 @@ if __name__ == '__main__':
 
 	gajim.log=log()
 
-	if gajim.config.get('emoticons_theme'):
-		print "emoticons"
+	gaj.Interface()
 
-	htmlview = HtmlTextView()
+	htmlview = ConversationTextview(None)
 
 	path_to_file = os.path.join(gajim.DATA_DIR, 'pixmaps', 'muc_separator.png')
 	# use this for hr
-	htmlview.focus_out_line_pixbuf =  gtk.gdk.pixbuf_new_from_file(path_to_file)
+	htmlview.tv.focus_out_line_pixbuf =  gtk.gdk.pixbuf_new_from_file(path_to_file)
 
 
 	tooltip = tooltips.BaseTooltip()
 	def on_textview_motion_notify_event(widget, event):
 		'''change the cursor to a hand when we are over a mail or an url'''
 		global change_cursor
-		pointer_x, pointer_y, spam = htmlview.window.get_pointer()
-		x, y = htmlview.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, pointer_x,
+		pointer_x, pointer_y = htmlview.tv.window.get_pointer()[0:2]
+		x, y = htmlview.tv.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, pointer_x,
 								   pointer_y)
-		tags = htmlview.get_iter_at_location(x, y).get_tags()
+		tags = htmlview.tv.get_iter_at_location(x, y).get_tags()
 		if change_cursor:
-			htmlview.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(
+			htmlview.tv.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(
 					 gtk.gdk.Cursor(gtk.gdk.XTERM))
 			change_cursor = None
-		tag_table = htmlview.get_buffer().get_tag_table()
-		over_line = False
+		tag_table = htmlview.tv.get_buffer().get_tag_table()
 		for tag in tags:
 			try:
 				if tag.is_anchor:
-					htmlview.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(
+					htmlview.tv.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(
 										gtk.gdk.Cursor(gtk.gdk.HAND2))
 					change_cursor = tag
 				elif tag == tag_table.lookup('focus-out-line'):
 					over_line = True
-			except: pass
+			except Exception:
+				pass
 
 		#if line_tooltip.timeout != 0:
 			# Check if we should hide the line tooltip
@@ -1035,32 +934,41 @@ if __name__ == '__main__':
 		#if over_line and not line_tooltip.win:
 		#	line_tooltip.timeout = gobject.timeout_add(500,
 		#		show_line_tooltip)
-		#	htmlview.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(
+		#	htmlview.tv.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(
 		#		gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))
 		#	change_cursor = tag
 
-	htmlview.connect('motion_notify_event', on_textview_motion_notify_event)
+	htmlview.tv.connect('motion_notify_event', on_textview_motion_notify_event)
 
 	def handler(texttag, widget, event, iter_, kind, href):
 		if event.type == gtk.gdk.BUTTON_PRESS:
 			print href
 
-	htmlview.html_hyperlink_handler = handler
+	htmlview.tv.html_hyperlink_handler = handler
 
-	htmlview.display_html('<div><span style="color: red; text-decoration:underline">Hello</span><br/>\n'
+	htmlview.print_real_text(None, xhtml='<div><span style="color: red; text-decoration:underline">Hello</span><br/>\n'
 						  '  <img src="http://images.slashdot.org/topics/topicsoftware.gif"/><br/>\n'
 						  '  <span style="font-size: 500%; font-family: serif">World</span>\n'
 						  '</div>\n')
-	htmlview.display_html('<hr />')
-	htmlview.display_html('''
+	htmlview.print_real_text(None, xhtml='<hr />')
+	htmlview.print_real_text(None, xhtml='''<body xmlns='http://www.w3.org/1999/xhtml'><p xmlns='http://www.w3.org/1999/xhtml'>a:b<a href='http://google.com/' xmlns='http://www.w3.org/1999/xhtml'>Google</a></p><br/></body>''')
+	htmlview.print_real_text(None, xhtml='''
+	 <body xmlns='http://www.w3.org/1999/xhtml'>
 	  <p style='font-size:large'>
-		<span style='font-style: italic'>O<span style='font-size:larger'>M</span>G</span>, 
+		<span style='font-style: italic'>O<span style='font-size:larger'>M</span>G</span>,
 		I&apos;m <span style='color:green'>green</span>
 		with <span style='font-weight: bold'>envy</span>!
 	  </p>
+	 </body>
 		''')
-	htmlview.display_html('<hr />')
-	htmlview.display_html('''
+	htmlview.print_real_text(None, xhtml='<hr />')
+	htmlview.print_real_text(None, xhtml='''
+	<body xmlns='http://www.w3.org/1999/xhtml'>
+		http://test.com/  testing links autolinkifying	
+	</body>
+		''')
+	htmlview.print_real_text(None, xhtml='<hr />')
+	htmlview.print_real_text(None, xhtml='''
 	<body xmlns='http://www.w3.org/1999/xhtml'>
 	  <p>As Emerson said in his essay <span style='font-style: italic; background-color:cyan'>Self-Reliance</span>:</p>
 	  <p style='margin-left: 5px; margin-right: 2%'>
@@ -1068,33 +976,33 @@ if __name__ == '__main__':
 	  </p>
 	</body>
 		''')
-	htmlview.display_html('<hr />')
-	htmlview.display_html('''
+	htmlview.print_real_text(None, xhtml='<hr />')
+	htmlview.print_real_text(None, xhtml='''
 	<body xmlns='http://www.w3.org/1999/xhtml'>
 	  <p style='text-align:center'>Hey, are you licensed to <a href='http://www.jabber.org/'>Jabber</a>?</p>
 	  <p style='text-align:right'><img src='http://www.jabber.org/images/psa-license.jpg'
-			  alt='A License to Jabber' 
-			  width='50%' height='50%' 
+			  alt='A License to Jabber'
+			  width='50%' height='50%'
 			  /></p>
 	</body>
 		''')
-	htmlview.display_html('<hr />')
-	htmlview.display_html('''
+	htmlview.print_real_text(None, xhtml='<hr />')
+	htmlview.print_real_text(None, xhtml='''
 	<body xmlns='http://www.w3.org/1999/xhtml'>
 	  <ul style='background-color:rgb(120,140,100)'>
 	   <li> One </li>
 	   <li> Two </li>
 	   <li> Three </li>
 	  </ul><hr /><pre style="background-color:rgb(120,120,120)">def fac(n):
-  def faciter(n,acc): 
+  def faciter(n,acc):
 	if n==0: return acc
 	return faciter(n-1, acc*n)
   if n&lt;0: raise ValueError('Must be non-negative')
   return faciter(n,1)</pre>
 	</body>
 		''')
-	htmlview.display_html('<hr />')
-	htmlview.display_html('''
+	htmlview.print_real_text(None, xhtml='<hr />')
+	htmlview.print_real_text(None, xhtml='''
 	<body xmlns='http://www.w3.org/1999/xhtml'>
 	 <ol style='background-color:rgb(120,140,100)'>
 	   <li> One </li>
@@ -1107,12 +1015,12 @@ if __name__ == '__main__':
 	   <li> Three </li></ol>
 	</body>
 		''')
-	htmlview.show()
+	htmlview.tv.show()
 	sw = gtk.ScrolledWindow()
 	sw.set_property('hscrollbar-policy', gtk.POLICY_AUTOMATIC)
 	sw.set_property('vscrollbar-policy', gtk.POLICY_AUTOMATIC)
 	sw.set_property('border-width', 0)
-	sw.add(htmlview)
+	sw.add(htmlview.tv)
 	sw.show()
 	frame = gtk.Frame()
 	frame.set_shadow_type(gtk.SHADOW_IN)
